import os
import sys
import unittest
import time
from datetime import datetime, date, timedelta

from p2.tools.prepare_db import setUpDB, tearDownDB
from p2.store.client import Client as StoreClient
from p2.driver.k8s import k8s

from p2.Scheduler import Scheduler
from p2.scheduler.Calendar import Calendar
from p2.scheduler.Test import P2test
from p2.logging import Logger

class TestSchedule(unittest.TestCase):

    def setUp(self):
        setUpDB()
        time.sleep(1)

        store_url = os.environ.get('P2_STORE_URL')
        self.client = StoreClient(store_url)
        experiments = self.client.get_experiments_as_dict()
        resources = self.client.get_k8s_resources(
            context='minikube',
            kind='namespace'
            )
        self.test = P2test(
            target = resources[0],
            experiment = experiments['exp_1'],
            expired = 30
            )

        logger = Logger(name='test_03_schedule')
        self.scheduler = Scheduler(logger=logger)
        self.log = logger.getLogger()

        self.k8s = k8s.Cron(context='minikube', namespace='default')
        self.k8s.delete_all_cronjobs()
        self.k8s.delete_all_cronjobs()
        time.sleep(1)

        def tearDown():
            tearDownDB()

    def test_schedule_this(self):
        self.k8s.delete_all_cronjobs()
        time.sleep(1)
        cronjobs_before = self.k8s.get_cronjobs()
        self.assertEqual(len(cronjobs_before), 0)

        campaigns = self.client.get_campaigns()

        if len(campaigns) == 0:
            self.fail("No campaigns found")

        self.scheduler.campaign_id=campaigns[0]['id']
        self.scheduler.cal = Calendar('minikube')
        isinvalid=True
        day = date.today()
        while isinvalid:
            day += timedelta(days=1)
            if self.scheduler.cal._day_valid(day):
                isinvalid = False
        self.log.info(f"Scheduling a test in day: {str(day)}")

        res = self.scheduler._schedule_this(
            test = self.test,
            day = day
            )
        self.assertIsNotNone(res)

        sessions = self.client.get_sessions(exp_id=res)
        self.assertEqual(len(sessions), 1)
        self.assertEqual(sessions[0]['exp_id'], res)
        self.assertEqual(sessions[0]['status'], 'scheduled')

        cronjobs_after = self.k8s.get_cronjobs()
        self.assertEqual(len(cronjobs_after), 1)

    def test_add_to_schedule(self):
        """First test:
        - ns3 (p3) => 30 days
        - exp_4 (w:0.5) 1 month ago, should be re-scheduled
        every 15 days...
        """
        resources = self.client.get_k8s_resources(name='ns3-int-nft')
        experiments = self.client.get_experiments(filename='exp_4')
        self.assertTrue(self.scheduler._add_to_schedule(
            target=resources[0],
            experiment=experiments[0]))
