import os
import sys
import unittest
import time
from datetime import datetime, date, timedelta

from p2.Scheduler import Scheduler
from p2.scheduler.Calendar import Calendar
from p2.tools.prepare_db import setUpDB, tearDownDB
from p2.store.client import Client as StoreClient
from p2.logging import Logger


class TestScheduler(unittest.TestCase):

    def setUp(self):
        setUpDB()
        time.sleep(1)

        logger = Logger(name=__name__)
        self.scheduler = Scheduler(logger=logger)

        store_url = os.environ.get('P2_STORE_URL')
        self.client = StoreClient(store_url)

        def tearDown():
            tearDownDB()

    def test_schedule_campaign(self):
        campaigns = self.client.get_campaigns()
        self.assertGreater(len(campaigns),0)
        res = self.scheduler.schedule_campaign(campaigns[0]['id'])
        self.assertEqual( len(res), 1 )

    def test_schedule_campaign_of_template(self):
        campaigns = self.client.select(
            table='test_campaigns',
            filter={
                'title': 'create remove dir'
            })
        self.assertGreater(len(campaigns),0)
        res = self.scheduler.schedule_campaign(campaigns[0]['id'])
        self.assertEqual( len(res), 1 )

    def test_min_interval(self):
        # this should give the minimum interval as no 
        # targets priority is set in config
        # so any component not there gets highest priority
        interval = self.scheduler.min_interval(
            target={'context': 'minikube', 'profile':{'priority':2}},
            experiment={'profile':{'weight':1.0}},
            status='completed'
            )
        self.assertEqual( interval, 7 )

    def test_get_last_time(self):
        campaigns = self.client.get_campaigns()
        self.assertGreater(len(campaigns),0)
        self.scheduler.campaign_id=campaigns[0]['id']
        self.scheduler.cal = Calendar('minikube')
        days=1
        tod = date.today() + timedelta(days=days)
        while not self.scheduler.cal._day_valid(tod):
            days += 1
            tod = date.today() + timedelta(days=days)

        last_time = self.scheduler._get_last_time(tod)
        print(last_time)
